@@ -4,10 +4,11 @@ use warnings qw(FATAL all NONFATAL misc);
 use 5.010; no if $] >= 5.017011, warnings => "experimental";
 
 use Carp qw(carp croak confess);
-our $VERSION = '0.100_003';
-#use mro 'c3';
+our $VERSION = '0.101_001';
+use mro 'c3';
 
 use Scalar::Util qw/weaken/;
+use List::MoreUtils qw/uniq/;
 
 #
 # YATT Internalへの Facade. YATT の初期化パラメータの保持者でもある。
@@ -23,7 +24,10 @@ use YATT::Lite::MFields qw/YATT
 	      cf_index_name
 	      cf_ext_public
 	      cf_ext_private
-	      cf_app_ns entns
+	      cf_app_ns
+	      entns
+	      cgen_class
+
 	      cf_app_name
 	      cf_debug_cgen cf_debug_parser cf_namespace cf_only_parse
 	      cf_special_entities cf_no_lineinfo cf_check_lineno
@@ -34,12 +38,17 @@ use YATT::Lite::MFields qw/YATT
 	      cf_info
 	      cf_lcmsg_sink
 	      cf_always_refresh_deps
+	      cf_no_mro_c3
 
 	      cf_default_lang
 
-	      cf_path2entns
+	      cf_entns2vfs_item
+	      cf_import
+              cf_match_argsroute_first
 	    /;
 
+use constant DEBUG => $ENV{DEBUG_YATT_LITE};
+
 MY->cf_mkaccessors(qw/app_name/);
 
 # Entities を多重継承する理由は import も継承したいから。
@@ -55,7 +64,7 @@ use YATT::Lite::Partial::AppPath;
 use YATT::Lite::Util qw/globref lexpand extname ckrequire terse_dump escape
 			set_inc ostream try_invoke list_isa symtab
 			look_for_globref
-			subname ckeval
+			subname ckeval ckrequire
 			secure_text_plain
 			define_const
 		       /;
@@ -79,14 +88,65 @@ sub after_new {
   $self->{cf_index_name} //= "";
   $self->{cf_ext_public} //= $self->default_ext_public;
   $self->{cf_ext_private} //= $self->default_ext_private;
+}
+
+sub _after_after_new {
+  (my MY $self) = @_;
   weaken($self->{cf_factory});
 }
 
 # XXX: kludge!
-sub create_neighbor {
+sub find_neighbor_yatt {
   (my MY $self, my ($dir)) = @_;
-  my MY $yatt = $self->{cf_factory}->load_yatt($dir);
-  $yatt->get_trans->root;
+  $self->{cf_factory}->load_yatt($dir);
+}
+sub find_neighbor_vfs {
+  (my MY $self, my ($dir)) = @_;
+  $self->find_neighbor_yatt($dir)->get_trans;
+}
+sub find_neighbor {
+  (my MY $self, my ($dir)) = @_;
+  $self->find_neighbor_vfs($dir)->root;
+}
+
+#
+# list all configs (named $name). (base first, then local one)
+# (useful to avoid config repeation)
+#
+sub cget_all {
+  (my MY $self, my $name) = @_;
+  (map($_->cget_all($name)
+       , $self->list_base_obj)
+   , lexpand($self->{"cf_$name"}));
+}
+
+sub list_base_obj {
+  (my MY $self) = @_;
+  map {
+    $self->find_neighbor_yatt($self->app_path_normalize($_))
+  } $self->list_base_dir;
+}
+
+sub list_base_dir {
+  (my MY $self) = @_;
+
+  my $base = $self->{cf_base} // do {
+    my %vfs = lexpand($self->{cf_vfs});
+    [map {
+      #
+      # Each element of $vfs{base} is either ARRAY (of vfs spec)
+      # or YATT::Lite::VFS::Dir object (instantiated from spec).
+      #
+      if (ref $_ eq 'ARRAY') {
+	my %vfs_base = @$_;
+	$vfs_base{dir};
+      } else {
+	$_->{cf_path};
+      }
+    } lexpand($vfs{base})];
+  };
+
+  lexpand($base);
 }
 
 #========================================
@@ -287,21 +347,33 @@ sub build_trans {
     (\@vfsspec
      , facade => $self
      , cache => $vfscache
+     , entns2vfs_item => $self->{cf_entns2vfs_item}
      , entns => $self->{entns}
      , @rest
-     # XXX: Should be more extensible.
-     , $self->cf_delegate_defined(qw/namespace base
-				     die_in_error tmpl_encoding
-				     debug_cgen debug_parser
-				     special_entities no_lineinfo check_lineno
-				     index_name
-				     ext_public
-				     ext_private
-				     rc_script
-				     lcmsg_sink
-				     only_parse
-				     always_refresh_deps
-				    /));
+     , $self->cf_delegate_defined($self->_cf_delegates));
+}
+
+sub _cf_delegates {
+  qw(namespace
+     base
+     die_in_error
+     tmpl_encoding
+     debug_cgen
+     debug_parser
+     special_entities
+     no_lineinfo
+     check_lineno
+     index_name
+     ext_public
+     ext_private
+     rc_script
+     lcmsg_sink
+     only_parse
+     always_refresh_deps
+     no_mro_c3
+     import
+     match_argsroute_first
+    )
 }
 
 sub _before_after_new {
@@ -310,6 +382,31 @@ sub _before_after_new {
   $self->{entns} = $self->ensure_entns($self->{cf_app_ns});
 }
 
+#========================================
+# Code generator class
+#========================================
+
+sub root_CGEN_perl () { 'YATT::Lite::CGen::Perl' }
+*CGEN_perl = *root_CGEN_perl; *CGEN_perl = *root_CGEN_perl;
+sub ensure_cgen_for {
+  my ($mypack, $type, $app_ns) = @_;
+  $mypack->ensure_supplns("CGEN_$type" => $app_ns);
+}
+
+sub get_cgen_class {
+  (my MY $self, my $type) = @_;
+  my $name = "CGEN_$type";
+  my $sub = $self->can("root_$name")
+    or croak "Unknown cgen class: $type";
+  $self->{cgen_class}{$type}
+    ||= $self->ensure_cgen_for($type, $self->{cf_app_ns});
+}
+
+sub is_default_cgen_ready {
+  (my MY $self) = @_;
+  $self->{cgen_class}{perl};
+}
+
 #========================================
 # Entity
 #========================================
@@ -320,45 +417,120 @@ sub root_EntNS { 'YATT::Lite::Entities' }
 # $app_ns に EntNS constant を追加する。
 # 複数回呼ばれた場合、既に定義済みの entns を返す
 
-sub ensure_entns {
-  my ($mypack, $app_ns, @baseclass) = @_;
-  my $entns = "${app_ns}::EntNS";
+sub should_use_mro_c3 {
+  (my MY $self_or_pack) = @_;
+  if (ref $self_or_pack) {
+    not $self_or_pack->{cf_no_mro_c3}
+  } else {
+    mro::get_mro($self_or_pack) eq 'c3';
+  }
+}
 
-  my $sym = do {no strict 'refs'; \*{$entns}};
+#========================================
+
+# These ns-related methods (ensure_...) are called as Class Methods.
+# This means you can't touch instance fields.
+
+# Old interface.
+# ensure_entns($app_ns, @base_entns)
+# returns EntNS for $app_ns with correct inheritance settings.
+#
+sub ensure_entns {
+  my ($mypack, $app_ns, @base_entns) = @_;
+  my $entns = $mypack->ensure_supplns(EntNS => $app_ns, \@base_entns
+				      , undef, +{no_fields => 1});
+  $entns;
+}
+
+# New interface.
+# ensure_supplns($kind, $app_ns, [@base_suppls], [@base_mains], {%opts})
+# returns ${app_ns}::${kind} with correct inheritance.
+#
+# [@base_suppls] gives base supplemental classes for this supplns.
+# [@base_mains] gives (not supplemental but) main classes for this.
+#
+# If both base_suppls and base_mains is empty, base_mains is derived
+# from current @ISA of $app_ns.
+#
+sub ensure_supplns {
+  my ($mypack, $kind, $app_ns, $base_suppls, $base_mains, $opts) = @_;
+
+  my $supplns = join("::", $app_ns, $kind);
+
+  my $sym = do {no strict 'refs'; \*{$supplns}};
   if (*{$sym}{CODE}) {
-    # croak "EntNS for $app_ns is already defined!";
-    return $entns;
+    # croak "$kind for $app_ns is already defined!";
+    return $supplns;
   }
 
-  # mro::set_mro($entns, 'c3'); # XXX: Should change to c3, but...
+  my $app_ns_filename = do {
+    my $sub = $app_ns->can("filename");
+    $sub ? ("(For path '".($sub->() // '')."')") : "";
+  };
+
+  print STDERR "# First ensure_supplns $kind for $app_ns $app_ns_filename: "
+    , terse_dump($base_suppls, $base_mains, $opts), "\n" if DEBUG;
+
+  if (not $base_suppls and not $base_mains) {
+    my @isa = list_isa($app_ns);
+    print STDERR "# app_ns $app_ns isa: "
+      , terse_dump(@isa), "\n" if DEBUG;
+
+    $base_mains = $mypack->should_use_mro_c3
+      ? [reverse @isa] : \@isa;
+  }
+
+  my @baseclass = (lexpand($base_suppls), map {
+    $mypack->ensure_supplns($kind => $_, undef, undef, $opts);
+  } lexpand($base_mains));
+
+  if ($mypack->should_use_mro_c3) {
+    print STDERR "# $kind - Set mro c3 for $supplns $app_ns_filename since $mypack uses c3\n" if DEBUG;
+    mro::set_mro($supplns, 'c3')
+  } else {
+    print STDERR "# $kind - Keep mro dfs for $supplns $app_ns_filename since $mypack uses dfs\n" if DEBUG;
+  }
 
   # $app_ns が %FIELDS 定義を持たない時(ex YLObjectでもPartialでもない)に限り、
   # YATT::Lite への継承を設定する
   unless (YATT::Lite::MFields->has_fields($app_ns)) {
     # XXX: $mypack への継承にすると、あちこち動かなくなるぜ？なんで？
+    print STDERR "# app_ns - Add ISA for '$app_ns' with fields: ",MY,"\n"
+      if DEBUG;
     YATT::Lite::MFields->add_isa_to($app_ns, MY)->define_fields($app_ns);
   }
 
-  unless (grep {$_->can("EntNS")} @baseclass) {
-    my $base = try_invoke($app_ns, 'EntNS') // $mypack->root_EntNS;
-    # print "insert base '$base' for entns $entns\n";
-    unshift @baseclass, $base;
+  unless (grep {$_->can($kind)} @baseclass) {
+    my $base = try_invoke($app_ns, $kind) // $mypack->can("root_$kind")->();
+    ckrequire($base);
+    print STDERR "# $kind - Set default base for $supplns <- ($base)\n" if DEBUG;
+    if ($mypack->should_use_mro_c3) {
+      push @baseclass, $base;
+    } else {
+      unshift @baseclass, $base;
+    }
   }
 
-  # print "entns $entns should inherits: @baseclass\n";
-  YATT::Lite::MFields->add_isa_to($entns, @baseclass);
+  do {
+    my @cls = uniq @baseclass;
+    print STDERR "# $kind - Add ISA for $supplns <- (@cls)\n" if DEBUG;
+    YATT::Lite::MFields->add_isa_to($supplns, @cls);
+  };
+  if (not $opts->{no_fields}) {
+    YATT::Lite::MFields->define_fields($supplns);
+  }
 
-  set_inc($entns, 1);
+  set_inc($supplns, 1);
 
-  # EntNS() を足すのは最後にしないと、再帰継承に陥る
+  # $kind() を足すのは最後にしないと、再帰継承に陥る
   unless (my $code = *{$sym}{CODE}) {
-    define_const($sym, $entns);
-  } elsif ((my $old = $code->()) ne $entns) {
-    croak "Can't add EntNS() to '$app_ns'. Already has EntNS as $old!";
+    define_const($sym, $supplns);
+  } elsif ((my $old = $code->()) ne $supplns) {
+    croak "Can't add $kind() to '$app_ns'. Already has $kind as $old!";
   } else {
     # ok.
   }
-  $entns
+  $supplns
 }
 
 sub list_entns {
@@ -387,7 +559,7 @@ sub define_Entity {
       my ($name, $sub) = @_;
       my $longname = join "::", $destns, "entity_$name";
       subname($longname, $sub);
-      print "defining entity_$name in $destns\n" if $ENV{DEBUG_ENTNS};
+      print STDERR "defining entity_$name in $destns\n" if DEBUG;
       *{globref($destns, "entity_$name")} = $sub;
     };
   }
@@ -403,6 +575,58 @@ sub define_Entity {
   return $destns;
 }
 
+#
+# Note about 'Action' registration mechanism in .htyattrc.pl
+#
+#  First, *globref() = $action is not enough. Because...
+#
+#  There are 2 places to hold actions.
+#    1. $YATT->{Action}      <= comes from *.ydo
+#    2. $vfs_folder->{Item}  <= comes from !yatt:action in templates
+#
+#  Action in .htyattrc.pl is, special case of 2.
+#  Since 2. is managed by yatt vfs, it must be wrapped by Action object
+#  so that $vfs->find_part_handler works well.
+#
+#
+#  This is bit complicated because .htyattrc.pl is loaded *BEFORE* $YATT
+#  is instantiated. This means "Action => name, $handler" can not touch $YATT
+#  at that time. So, I need to delay actual registration until $YATT is created.
+#
+#  To achieve this, $handler is registered first in %Actions of caller,
+#  then installed into actual vfs.
+#
+# Also note: loading of .htyattrc.pl is handled by Factory.
+#
+sub ACTION_DICT_SYM () {'Actions'}
+sub define_Action {
+  my ($myPack, $opts, $callpack) = @_;
+
+  *{globref($callpack, ACTION_DICT_SYM)} = my $action_dict = +{};
+
+  *{globref($callpack, 'Action')} = sub {
+    my ($name, $sub) = @_;
+    my @caller = my ($callpack, $filename, $lineno) = caller;
+    if (defined (my $old = $action_dict->{$name})) {
+      croak "Duplicate definition of Action '$name'! previously"
+	." at $old->[1][1] line $old->[1][2]\n new at $filename line $lineno\n";
+    }
+    $action_dict->{$name} = [$sub, \@caller];
+  };
+}
+
+sub setup_rc_actions {
+  (my $self) = @_;
+  my $glob = look_for_globref($self, ACTION_DICT_SYM)
+    or return;
+  my $dict = *{$glob}{HASH};
+
+  my $vfs = $self->get_vfs;
+  foreach my $name (keys %$dict) {
+    $vfs->add_root_action_handler($name, @{$dict->{$name}});
+  }
+}
+
 # ここで言う Object系とは、
 #   YATT::Lite::Object を継承してるか、
 #   又は既に %FIELDS が定義されている class
@@ -461,6 +685,7 @@ sub default_lang {
 #========================================
 foreach
   (qw/find_part
+      find_part_from_entns
       find_file
       find_product
       find_renderer
@@ -514,14 +739,28 @@ sub YATT::Lite::EntNS::entity_mkhidden {
   my ($this) = shift;
   \ join "\n", map {
     my $name = $_;
-    my $esc = escape($name);
     map {
-      sprintf(qq|<input type="hidden" name="%s" value="%s"/>|
-	      , $esc, escape($_));
-    } $CON->param($name);
+      my $v = $_;
+      if (ref $v eq 'HASH') {
+        map {
+          _hidden_input(escape($name."[$_]"), $v->{$_});
+        } keys %$v;
+      } elsif (ref $v eq 'ARRAY') {
+        map {
+          _hidden_input(escape($name."[]"), $_);
+        } @$v;
+      } else {
+        _hidden_input(escape($name), $v);
+      }
+    } $CON->multi_param($name);
   } @_;
 };
 
+sub _hidden_input {
+  sprintf(qq|<input type="hidden" name="%s" value="%s">|
+          , $_[0], escape($_[1]));
+}
+
 sub YATT::Lite::EntNS::entity_file_rootname {
   my ($this, $fn) = @_;
   $fn //= $CON->file();
