@@ -1,8 +1,12 @@
 package YATT::Lite::CGen; sub MY () {__PACKAGE__}
 use strict;
 use warnings qw(FATAL all NONFATAL misc);
+use Carp;
+
+use constant DEBUG_REBUILD => $ENV{DEBUG_YATT_REBUILD};
+
 use base qw(YATT::Lite::VarMaker);
-use fields qw/curtmpl curwidget curtoks
+use YATT::Lite::MFields qw/curtmpl curwidget curtoks
 	      altgen needs_escaping depth
 	      cf_cgen_loader
 	      cf_only_parse
@@ -12,10 +16,18 @@ use fields qw/curtmpl curwidget curtoks
 	      cf_lcmsg_sink
 	     /;
 
-use YATT::Lite::Core qw(Template Part);
+use YATT::Lite::Core qw(Template Part Folder);
 use YATT::Lite::Constants;
 use YATT::Lite::Util qw(callerinfo numLines);
-use Carp;
+
+sub ensure_generated_for_folders {
+  (my MY $self, my $spec) = splice @_, 0, 2;
+  foreach my Folder $folder (@_) {
+    if ($folder->can_generate_code) {
+      $self->ensure_generated($spec, $folder);
+    }
+  }
+}
 
 sub ensure_generated {
   (my MY $self, my $spec, my Template $tmpl) = @_;
@@ -24,10 +36,16 @@ sub ensure_generated {
   return if defined $tmpl->{product}{$type};
   local $self->{depth} = 1 + ($self->{depth} // 0);
   my $pkg = $tmpl->{product}{$type} = $tmpl->{cf_entns};
+  if (not defined $tmpl->{product}{$type}) {
+    croak "package for product $type of $tmpl->{cf_path} is not defined!";
+  } else {
+    print STDERR "# generating $pkg for $type code of $tmpl->{cf_path}\n"
+      if DEBUG_REBUILD;
+  }
   $self->{cf_parser}->parse_body($tmpl)
     if not $kind or not $self->{cf_only_parse}
       or $self->{cf_only_parse}{$kind};
-  $self->setup_inheritance($tmpl);
+  $self->setup_inheritance_for($spec, $tmpl);
   my @res = $self->generate($tmpl, $kind);
   if (my $sub = $self->{cf_sink}) {
     $sub->({folder => $tmpl, package => $pkg, kind => 'body'
@@ -58,7 +76,10 @@ sub generate {
   } @{$tmpl->{partlist}});
 }
 
-sub setup_inheritance {}
+sub setup_inheritance_for {
+  (my MY $self, my $spec, my Template $tmpl) = @_;
+  $self->ensure_generated_for_folders($spec, $tmpl->list_base);
+}
 
 #========================================
 sub altgen {
@@ -142,7 +163,8 @@ sub cut_next_nl {
   my MY $self = shift;
   # undef は返したくないので。
   return wantarray ? () : ''
-    unless @{$self->{curtoks}} and $self->{curtoks}[0] =~ /^\r?\n$/;
+    unless $self->{curtoks}
+    and @{$self->{curtoks}} and $self->{curtoks}[0] =~ /^\r?\n$/;
   return wantarray ? () : ''
     if @{$self->{curtoks}} == 1; # 最後の一個の改行は、残す。これは "}\n" のため
   $self->{curline}++;
