@@ -1,9 +1,12 @@
 package YATT::Lite::CGen::Perl;
 use strict;
 use warnings qw(FATAL all NONFATAL misc);
+use mro 'c3';
 
 require 5.010; # For named capture.
 
+use constant DEBUG_MRO => $ENV{DEBUG_YATT_MRO};
+
 use YATT::Lite::Core qw(Folder Template Part Widget Action);
 use YATT::Lite::Constants;
 
@@ -17,7 +20,8 @@ use YATT::Lite::Constants;
   #========================================
   package YATT::Lite::CGen::Perl; sub MY () {__PACKAGE__}
   use base qw(YATT::Lite::CGen);
-  use YATT::Lite::Util qw(lexpand numLines globref terse_dump);
+  use YATT::Lite::MFields;
+  use YATT::Lite::Util qw(lexpand numLines globref terse_dump catch);
   use Carp;
   #========================================
   sub list_inheritance {
@@ -26,6 +30,9 @@ use YATT::Lite::Constants;
     my %dup;
     map {
       my Folder $f = $_;
+      unless (defined $f->{cf_entns}) {
+	die "BUG: EntNS is empty for ".terse_dump($f->{cf_name})."!";
+      }
       if ($dup{$f->{cf_entns}}++) {
 	()
       } else {
@@ -33,8 +40,8 @@ use YATT::Lite::Constants;
       }
     } $tmpl->list_base
   }
-  sub setup_inheritance {
-    (my MY $self, my Template $tmpl) = @_;
+  sub setup_inheritance_for {
+    (my MY $self, my $spec, my Template $tmpl) = @_;
     unless (defined $tmpl->{cf_entns}) {
       die "BUG: EntNS is empty for '$tmpl->{cf_name}'!";
     }
@@ -43,15 +50,28 @@ use YATT::Lite::Constants;
     unless (defined $glob) {
       die "BUG: ISA glob for '$tmpl->{cf_name}' is empty!";
     }
+    $self->ensure_generated_for_folders($spec, $tmpl->list_base);
     my @isa = $self->list_inheritance($tmpl);
     if (grep {not defined} @isa) {
       die "BUG: ISA for '$tmpl->{cf_name}' contains undef!";
     }
-    *$glob = \@isa;
+    if (my $err = catch {
+      *$glob = \@isa;
+    }) {
+      die $self->generror("Can't set ISA for '%s' as [%s]: %s"
+			  , $tmpl->{cf_name}
+			  , join(", ", @isa)
+			  , $err
+			);
+    }
   }
   sub generate_inheritance {
     (my MY $self, my Template $tmpl) = @_;
-    sprintf q{our @ISA = qw(%s); }, join " ", $self->list_inheritance($tmpl);
+    my @isa = $self->list_inheritance($tmpl);
+    my $mro = mro::get_mro($tmpl->{cf_entns});
+    print STDERR "($mro) [$tmpl->{cf_path}] $tmpl->{cf_entns}::ISA = @isa\n"
+      if DEBUG_MRO;
+    sprintf q{use mro '%s'; our @ISA = qw(%s); }, $mro, join " ", @isa;
   }
   #========================================
   sub generate_preamble {
@@ -103,13 +123,24 @@ use YATT::Lite::Constants;
   sub generate_action {
     (my MY $self, my Action $action) = @_;
     # XXX: 改行の調整が必要。
+    my @src = ($self->sync_curline($action->{cf_startln})
+               , "sub $$action{cf_name} {");
     my $src = $self->{curtmpl}->source_substr
       ($action->{cf_bodypos}, $action->{cf_bodylen});
+
+    if (lexpand($action->{arg_order})
+        or $src !~ m{^([\ \t\r\n]*)my\s*\([^;\)]+\)\s*=\s*\@_\s*;}) {
+      # If an action has no arguments
+      # and its source doesn't start with my (...) = @_;,
+      # insert preamble and getargs.
+      push @src, $self->gen_preamble($action)
+        , $self->gen_getargs($action, not $action->{cf_implicit});
+    }
+
     my $has_nl = $src =~ s/\r?\n\Z//;
     $self->{curline} = $action->{cf_bodyln} + numLines($src)
       + ($has_nl ? 1 : 0);
-    sprintf "sub %s {%s}\n"
-      , $action->{cf_name}, $src;
+    (@src, $src, "}");
   }
   #========================================
   sub gen_preamble {q{ my ($this, $CON) = splice @_, 0, 2;}}
@@ -304,8 +335,13 @@ use YATT::Lite::Constants;
       my $err = $self->generror(q{No such widget <%s>}, $wname);
       die $err;
     };
+
     $self->ensure_generated(perl => my Template $tmpl = $widget->{cf_folder});
-    my $that = $tmpl == $self->{curtmpl} ? '$this' : $tmpl->{cf_entns};
+    my $use_this = $tmpl == $self->{curtmpl};
+    unless ($use_this) {
+      $self->{curtmpl}->add_dependency($wname, $tmpl);
+    }
+    my $that = $use_this ? '$this' : $tmpl->{cf_entns};
     \ sprintf(q{%s->render_%s($CON, %s)}
 	      , $that, $widget->{cf_name}
 	      , $self->gen_putargs($widget, $node)
@@ -870,9 +906,14 @@ sub feed_arg_spec {
 
   sub macro_block {
     (my MY $self, my $node) = @_;
-    local $self->{scope} = $self->mkscope({}, $self->{scope});
     my ($path, $body, $primary, $head, $foot) = nx($node);
-    local $self->{curtoks} = [@{argValue($body)}];
+    $self->macro_scoped_block_of_tokens(+{}, @{argValue($body)});
+  }
+
+  sub macro_scoped_block_of_tokens {
+    (my MY $self, my ($scope, @tokens)) = @_;
+    local $self->{scope} = $self->mkscope($scope, $self->{scope});
+    local $self->{curtoks} = \@tokens;
     \ ('{'.$self->as_print('}'));
   }
 }
