@@ -174,6 +174,14 @@ sub cf_pairs {
 
 #========================================
 
+# Just a wrapper for die.
+sub raise_response {
+  my ($con, $response) = @_;
+  die $response;
+}
+
+#========================================
+
 sub as_error {
   my PROP $prop = prop(my $glob = shift);
   $prop->{is_error} = 1;
@@ -185,6 +193,12 @@ sub as_error {
   $glob;
 }
 
+sub error_with_status {
+  my ($glob, $code) = splice @_, 0, 2;
+  $glob->configure(status => $code)
+    ->raise(error => incr_opt(depth => \@_), @_);
+}
+
 sub error {
   # XXX: as_error?
   shift->raise(error => incr_opt(depth => \@_), @_);
