@@ -2,6 +2,9 @@ package YATT::Lite::Core; sub MY () {__PACKAGE__}
 use strict;
 use warnings qw(FATAL all NONFATAL misc);
 use Carp;
+
+use constant DEBUG_REBUILD => $ENV{DEBUG_YATT_REBUILD};
+
 use parent qw(YATT::Lite::VFS);
 use YATT::Lite::MFields qw/cf_namespace cf_debug_cgen cf_no_lineinfo cf_check_lineno
 			   cf_index_name
@@ -11,8 +14,9 @@ use YATT::Lite::MFields qw/cf_namespace cf_debug_cgen cf_no_lineinfo cf_check_li
 	      cf_die_in_error cf_error_handler
 	      cf_special_entities
 	      cf_lcmsg_sink
+              cf_match_argsroute_first
 
-	      cgen_class
+	      n_compiles
 	    /;
 use YATT::Lite::Util;
 use YATT::Lite::Constants;
@@ -43,14 +47,43 @@ use YATT::Lite::Breakpoint ();
 
      , [Template => -base => MY->File
 	, -alias => 'vfs_file'
+	, -constants => [[can_generate_code => 1]]
 	, -fields => [qw(product parse_ok cf_mtime cf_utf8 cf_age
 			 cf_usage cf_constants
 			 cf_ignore_trailing_newlines
 			 cf_subroutes
-		       )]]
+		      )]]
+
+     , [ParsingState => -fields => [qw(startln endln
+				       startpos curpos
+				       cf_path
+				       )]]
     );
 
-  # folder の weaken は parser がしてる。
+  sub YATT::Lite::Core::Part::public_name {
+    (my Part $part) = @_;
+    $part->{cf_name};
+  }
+  sub YATT::Lite::Core::Action::public_name {
+    (my Action $action) = @_;
+    substr($action->{cf_name}, 3);
+  }
+  sub YATT::Lite::Core::Part::method_name {...}
+  sub YATT::Lite::Core::Widget::method_name {
+    (my Widget $widget) = @_;
+    "render_$widget->{cf_name}";
+  }
+  sub YATT::Lite::Core::Action::method_name {
+    (my Action $action) = @_;
+    $action->{cf_name};
+  }
+
+  sub YATT::Lite::Core::Part::configure_folder {
+    (my Part $part, my Folder $folder) = @_;
+    Scalar::Util::weaken($part->{cf_folder} = $folder);
+    # die "Can't weaken!" unless Scalar::Util::isweak($part->{cf_folder});
+  }
+
 #  sub YATT::Lite::Core::Part::source {
 #    (my Part $part) = @_;
 #    join "", map {ref $_ ? "\n" x $$_[0] : $_} @{$part->{source}};
@@ -93,13 +126,15 @@ use YATT::Lite::Breakpoint ();
   }
 
   sub YATT::Lite::Core::Part::reorder_hash_params {
-    (my Widget $widget, my ($params)) = @_;
+    (my Widget $widget, my ($orig_params)) = @_;
+    my $params = +{%$orig_params};
     my @params;
     foreach my $name (map($_ ? @$_ : (), $widget->{arg_order})) {
       push @params, delete $params->{$name};
     }
     if (keys %$params) {
-      die "Unknown args for $widget->{cf_name}: " . join(", ", keys %$params);
+      die "Unknown args for $widget->{cf_name}: " . join(", ", keys %$params)
+	. "\n";
     }
     wantarray ? @params : \@params;
   }
@@ -109,8 +144,9 @@ use YATT::Lite::Breakpoint ();
     $list ||= [];
     foreach my $name ($cgi->param) {
       next unless $name =~ /^[a-z]\w*$/i;
+      my $wname = $widget->{cf_name} ? " for widget '$widget->{cf_name}'" : "";
       my $argdecl = $widget->{arg_dict}{$name}
-	or die "Unknown args for widget '$widget->{cf_name}': $name";
+	or die "Unknown args$wname: $name\n";
       my @value = $cgi->multi_param($name);
       $list->[$argdecl->argno] = $argdecl->type->[0] eq 'list'
 	? \@value : $value[0];
@@ -137,6 +173,55 @@ sub create_file {
   $vfs->Template->new(path => $spec, @_);
 }
 
+#
+# called from <!yatt:base>
+#
+sub declare_base {
+  (my MY $vfs, my ParsingState $state, my Template $tmpl, my ($ns, @args)) = @_;
+
+  unless (@args) {
+    $vfs->synerror($state, q{No base arg});
+  }
+
+  my $base = $tmpl->{cf_base} //= [];
+  if (@$base) {
+    $vfs->synerror($state, "Duplicate base decl! was=%s, new=%s"
+		   , terse_dump($base), terse_dump(\@args));
+  }
+
+  foreach my $att (@args) {
+    my $type = $vfs->node_type($att);
+
+    $type == TYPE_ATT_TEXT
+      or $vfs->synerror($state, q{Not implemented base decl type: %s}, $att);
+
+    nonempty(my $fn = $vfs->node_value($att))
+      or $vfs->synerror($state, q{base spec is empty!});
+
+    if ($vfs->{on_memory}) {
+      my $o = $vfs->find_file($fn)
+	or $vfs->synerror($state, q{No such base path: %s}, $fn);
+      push @$base, $o;
+    } else {
+      defined(my $realfn = $vfs->resolve_path_from($tmpl, $fn))
+	or $vfs->synerror($state, q{Can't find object path for: %s}, $fn);
+
+      -e $realfn
+	or $vfs->synerror($state, q{No such base path: %s}, $realfn);
+
+      push @$base, $vfs->find_neighbor_type(undef, $realfn);
+    }
+  }
+}
+
+sub synerror {
+  (my MY $vfs, my ParsingState $state, my ($fmt, @opts)) = @_;
+  my $opts = {depth => 2};
+  $opts->{tmpl_file} = $state->{cf_path} if $state->{cf_path};
+  $opts->{tmpl_line} = $state->{startln} if $state->{startln};
+  die $vfs->error($opts, $fmt, @opts);
+}
+
 #========================================
 {
   sub Parser {
@@ -160,7 +245,9 @@ sub create_file {
     # $self->{parser} ||=
       $self->Parser->new
 	(vfs => $self, $self->cf_delegate
-	 (qw(namespace special_entities)
+	 (qw(namespace special_entities
+             match_argsroute_first
+          )
 	  , [debug_parser => 'debug']
 	  , [tmpl_encoding => 'encoding']
 	 )
@@ -254,7 +341,7 @@ sub create_file {
       $tmpl = $self->find_file($partName)
 	or ($ignore_error and return)
 	  or croak "No such template file: $partName";
-      $part = $tmpl->{Item}{$itemKey}
+      $part = $tmpl->{Item}{$itemKey} || $self->find_part_from($tmpl, $itemKey)
 	or ($ignore_error and return)
 	  or croak "No such $kind in file $partName: $pureName";
     }
@@ -274,6 +361,28 @@ sub create_file {
   sub _itemKey_page { shift; ($_[0], "render_$_[0]") }
   sub _itemKey_action { shift; ("do_$_[0]") x 2; }
 
+  #
+  # Action name => sub {}
+  #
+  sub add_root_action_handler {
+    (my MY $self, my ($name, $sub, $callinfo)) = @_;
+    my Folder $root = $self->{root};
+
+    my ($callpack, $filename, $lineno) = @$callinfo;
+
+    # XXX: This means do_$A.yatt will conflict with "Action $A" in .htyattrc.pl
+    my $action_name = "do_$name";
+
+    *{globref($root->{cf_entns}, $action_name)} = $sub;
+
+    $root->{Item}{$action_name}
+      = $self->Action->new(name => $action_name, kind => 'action'
+			   , folder => $root
+			   , startln => $lineno
+			 );
+
+  }
+
   sub find_renderer {
     my MY $self = shift;
     my ($part, $sub, $pkg) = $self->find_part_handler(@_)
@@ -284,15 +393,7 @@ sub create_file {
   # DirHandler INST 固有 CGEN_perl の生成
   sub get_cgen_class {
     (my MY $self, my $type) = @_;
-    my $sub = $self->can("cgen_$type")
-      || carp "Unknown product type: $type";
-    $self->{cgen_class}{$type} ||= do {
-      my $cg_base = $sub->();
-      # XXX: ref($facade) が INST 固有に成ってなかったら？
-      my $instpkg = ref($self->{cf_facade})."::CGEN_$type";
-      ckeval(qq|package $instpkg; use base qw($cg_base)|);
-      $instpkg;
-    };
+    $self->{cf_facade}->get_cgen_class($type);
   }
 
   # XXX: Action only コンパイルは？
@@ -309,7 +410,19 @@ sub create_file {
 	 , parser => $self->get_parser
 	 , sink => $opts{sink} || sub {
 	   my ($info, @script) = @_;
-	   print @script, "\n" if $self->{cf_debug_cgen};
+	   if (not $self->{cf_debug_cgen}) {
+	   } else {
+	     my Template $real = $info->{folder};
+	     print STDERR "# compiling $type code of $real->{cf_path}\n";
+	     if ($self->{cf_debug_cgen} >= 2) {
+	       print STDERR "#--BEGIN--\n";
+	       print STDERR @script, "\n";
+	       print STDERR "#--END--\n\n"
+	     }
+	   }
+	   #
+	   $self->{n_compiles}++;
+
 	   ckeval(@script);
 	 });
       # 二重生成防止のため、代入自体は ensure_generated の中で行う。
@@ -380,13 +493,26 @@ sub create_file {
   }
   sub YATT::Lite::Core::Template::refresh {
     (my Template $tmpl, my MY $self) = @_;
+
+    my $old_product = $tmpl->{product};
+
     if ($tmpl->{cf_path}) {
+      printf STDERR "template_refresh(%s)\n", $tmpl->{cf_path} if DEBUG_REBUILD;
       my $mtime = stat_mtime($tmpl->{cf_path});
       unless (defined $mtime) {
+	printf STDERR " => deleted\n" if DEBUG_REBUILD;
 	return; # XXX: ファイルが消された
       } elsif (defined $tmpl->{cf_mtime} and $tmpl->{cf_mtime} >= $mtime) {
+	if (DEBUG_REBUILD) {
+	  printf STDERR " => use cached. mtime(was=$tmpl->{cf_mtime}"
+	    .", now=$mtime) for tmpl=$tmpl\n";
+	}
+	$self->refresh_deps_for($tmpl) if $self->{cf_always_refresh_deps};
 	return; # timestamp は、キャッシュと同じかむしろ古い
       }
+      if (DEBUG_REBUILD) {
+	printf STDERR " => found update. mtime($mtime) for tmpl=$tmpl\n";
+      }
       $tmpl->{cf_mtime} = $mtime;
       my $parser = $self->get_parser;
       # decl のみ parse.
@@ -404,6 +530,14 @@ sub create_file {
     } else {
       return;
     }
+
+    # $tmpl->YATT::Lite::VFS::Folder::vivify_base_descs($self);
+
+    # If there was products, rebuild it too.
+    foreach my $type ($old_product ? keys %$old_product : ()) {
+      $self->find_product($type => $tmpl);
+    }
+
     $tmpl;
   }
   sub YATT::Lite::Core::Widget::fixup {
@@ -428,11 +562,6 @@ sub create_file {
   }
 }
 
-sub find_template_from_package {
-  (my MY $self, my $pkg) = @_;
-  $self->{pkg2folder}{$pkg};
-}
-
 use YATT::Lite::Breakpoint ();
 YATT::Lite::Breakpoint::break_load_core();
 
