@@ -1,9 +1,9 @@
 package YATT::Lite::Entities;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 
-#use mro 'c3';
+use mro 'c3';
 # XXX: 残念ながら、要整理。
 
 require YATT::Lite::MFields;
@@ -100,13 +100,24 @@ sub entity_breakpoint {
   &YATT::Lite::Breakpoint::breakpoint();
 }
 
+sub entity_param {
+  shift;
+  $CON->param(@_);
+}
+
 sub entity_concat {
   my $this = shift;
   join '', @_;
 }
 
-# coalesce
-*entity_coalesce = *entity_default; *entity_coalesce = *entity_default;
+sub entity_coalesce {
+  my $this = shift;
+  foreach my $str (@_) {
+    return $str if defined $str;
+  }
+  '';
+}
+
 sub entity_default {
   my $this = shift;
   foreach my $str (@_) {
