@@ -4,7 +4,12 @@ use warnings qw(FATAL all NONFATAL misc);
 use parent qw(YATT::Lite::Object);
 use constant DEBUG_VERBOSE => $ENV{YATT_DEBUG_VERBOSE};
 
+use Exporter qw/import/;
+our @EXPORT_OK = qw/Error/;
+our @EXPORT = @EXPORT_OK;
+
 use YATT::Lite::MFields qw/cf_file cf_line cf_tmpl_file cf_tmpl_line
+			   cf_http_status_code
 	      cf_backtrace
 	      cf_reason cf_format cf_args/;
 use overload qw("" message
@@ -47,6 +52,9 @@ sub reason {
 	  }
 	} $error->{cf_format}, lexpand($error->{cf_args})).")";
     }
+    BEGIN {
+      warnings->unimport(qw/redundant/) if $] >= 5.021002; # for sprintf
+    }
     sprintf $error->{cf_format}, map {
       defined $_ ? $_ : '(undef)'
     } lexpand($error->{cf_args});
