@@ -4,15 +4,22 @@ use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 use YATT::Lite::Breakpoint;
 sub MY () {__PACKAGE__}
+use mro 'c3';
+
+use constant DEBUG_FACTORY => $ENV{DEBUG_YATT_FACTORY};
+use constant DEBUG_REFCNT => $ENV{DEBUG_YATT_REFCNT};
+use if DEBUG_REFCNT, B => qw/svref_2object/;
 
 use 5.010;
 use Scalar::Util qw(weaken);
 use Encode qw/decode/;
 
-use parent qw/YATT::Lite::NSBuilder File::Spec/;
+use parent qw/File::Spec YATT::Lite::NSBuilder/;
 use File::Path ();
 use File::Basename qw/dirname/;
 
+use YATT::Lite::PSGIEnv;
+
 # Note: Definition of default values are not yet gathered here.
 # Some are in YATT::Lite, others are in YATT::Lite::Core, CGen.. and so on.
 
@@ -66,6 +73,7 @@ use YATT::Lite::MFields
 
  , qw/
        cf_allow_missing_dir
+       cf_no_preload_app_base
 
        tmpldirs
        loc2yatt
@@ -76,10 +84,29 @@ use YATT::Lite::MFields
 
        tmpl_cache
 
+       path2entns
+       entns2vfs_item
+
        cf_debug_cgen
 
        cf_only_parse
        cf_config_filetypes
+
+       cf_dont_map_args
+       cf_dont_debug_param
+       cf_always_refresh_deps
+       cf_no_mro_c3
+
+       cf_special_entities
+       cf_default_lang
+       cf_no_lineinfo
+       cf_debug_parser
+       cf_check_lineno
+
+       _outer_psgi_app
+       _my_psgi_app
+
+       cf_match_argsroute_first
      /);
 
 use YATT::Lite::Util::AsBase;
@@ -90,6 +117,7 @@ use YATT::Lite::Util qw/lexpand globref untaint_any ckrequire dofile_in
 			psgi_error
 			globref_default
 			define_const
+			terse_dump
 		       /;
 
 use YATT::Lite::XHF ();
@@ -175,20 +203,57 @@ sub load_factory_for_psgi {
 
 #========================================
 
+my ($_n_created, $_n_destroyed);
+sub n_created {$_n_created}
+sub n_destroyed {$_n_destroyed}
 {
-  my %sub2app;
-  sub to_app {
-    my ($self, $cascade, @fallback) = @_;
-    $self->prepare_app;
-    my $sub = sub { $self->call(@_) };
-    $sub2app{$sub} = $self; weaken($sub2app{$sub});
-    if ($cascade) {
-      $sub2app{$cascade} = $self; weaken($sub2app{$cascade});
-      $cascade->add($sub, @fallback);
-      $cascade->to_app;
-    } else {
-      $sub;
+  our %sub2self;
+  DESTROY {
+    (my MY $self) = @_;
+    print STDERR "DESTROY $self\n" if DEBUG_FACTORY;
+    delete $self->{_my_psgi_app};
+    if (my $outer = delete $self->{_outer_psgi_app}) {
+      delete $sub2self{$outer};
     }
+    ++$_n_destroyed;
+  };
+  sub to_app_and_forget {
+    (my MY $self) = @_;
+    my $sub = $self->to_app;
+    delete $self->{_my_psgi_app};
+    delete $self->{_outer_psgi_app};
+    delete $sub2self{$sub};
+    $sub;
+  }
+  sub to_app {
+    (my MY $self) = @_;
+    if (@_ >= 2) {
+      croak "cascade support is dropped.Use wrapped_by(builder {}) instead.";
+    }
+    $self->{_outer_psgi_app} // do {
+      if (my $old = delete $self->{_my_psgi_app}) {
+        delete $sub2self{$old};
+      }
+      $self->prepare_app;
+      my $sub = sub { $self->call(@_) };
+      $self->{_my_psgi_app} = $sub;
+      weaken($self->{_my_psgi_app}) if not $want_object;
+      weaken($sub2self{$sub} = $self);
+      print STDERR "to_app($self) returned $sub\n" if DEBUG_FACTORY;
+      $sub;
+    };
+  }
+  sub wrapped_by {
+    my ($self, $outer_app) = @_;
+    unless ($self->{_my_psgi_app}) {
+      croak "wrapped_by is called without calling Site->to_app";
+    }
+    delete $sub2self{$self->{_my_psgi_app}};
+    $self->{_outer_psgi_app} = $outer_app;
+    $sub2self{$outer_app} = $self;
+    weaken($self->{_outer_psgi_app}) if not $want_object;
+    weaken($sub2self{$outer_app});
+    $outer_app;
   }
   sub load_psgi_script {
     my ($pack, $fn) = @_;
@@ -196,7 +261,7 @@ sub load_factory_for_psgi {
     local $0 = $fn;
     my $sub = $pack->sandbox_dofile($fn);
     if (ref $sub eq 'CODE') {
-      $sub2app{$sub};
+      $sub2self{$sub};
     } elsif ($sub->isa($pack) or $sub->isa(MY)) {
       $sub;
     } else {
@@ -252,9 +317,33 @@ sub find_factory_script {
 
 #========================================
 
+sub new {
+  my ($class) = shift;
+  my MY $self = $class->SUPER::new(@_);
+  $self->preload_app_base unless $self->{cf_no_preload_app_base};
+  ++$_n_created;
+  $self;
+}
+
+#
+# preload app_base (to avoid potential double-loading bug for base loading)
+#
+sub preload_app_base {
+  (my MY $self) = @_;
+
+  foreach my $dir (lexpand($self->{cf_app_base})) {
+    next if $dir =~ m{^::};
+    $self->load_yatt($self->app_path_expand($dir));
+  }
+}
+
 sub init_app_ns {
   (my MY $self) = @_;
   $self->SUPER::init_app_ns;
+
+  # EntNS is initialized here.
+  # Note: CGEN_perl is not initialized here and delayed until it is required.
+  # This helps to avoid loading CGen::Perl for *.ydo in CGI.
   $self->{default_app}->ensure_entns($self->{app_ns});
 }
 
@@ -308,8 +397,19 @@ sub _after_after_new {
   $self->{tmpldirs} = [];
   if (my $dir = $self->{cf_doc_root}) {
     push @{$self->{tmpldirs}}, $dir;
+    my $refcnt;
+    if (DEBUG_REFCNT) {
+      $refcnt = svref_2object($self)->REFCNT;
+    }
     $self->get_yatt('/');
+    if (DEBUG_REFCNT) {
+      if (svref_2object($self)->REFCNT != $refcnt) {
+        croak "Reference count of $self is increased from $refcnt to "
+          . svref_2object($self)->REFCNT . "!";
+      }
+    }
   }
+  $self;
 }
 
 #========================================
@@ -414,6 +514,29 @@ sub rebuild_psgi_mount {
   $self->{loc2psgi_re} = qr{^(?:$all)(?:/|$)};
 }
 
+sub psgi_file_app {
+  my ($pack, $path) = @_;
+  require Plack::App::File;
+  Plack::App::File->new(root => $path)->to_app;
+}
+
+sub mount_static {
+  (my MY $self, my ($location, $realpath)) = @_;
+  my $app = ref $realpath eq 'CODE' ? $realpath
+    : $self->psgi_file_app($realpath);
+  $self->mount_psgi
+    ($location, sub {
+       (my Env $env) = @_;
+       local $env->{PATH_INFO} = _trim_prefix($env->{PATH_INFO}, $location);
+       $app->($env);
+     });
+}
+
+sub _trim_prefix {
+  substr($_[0], length($_[1]));
+}
+
+
 #========================================
 
 sub mount_action {
@@ -526,8 +649,15 @@ sub get_yatt {
 
 # phys-path => yatt
 
+*get_dirhandler = *load_yatt; *get_dirhandler = *load_yatt;
+
 sub load_yatt {
   (my MY $self, my ($path, $basedir, $visits, $from)) = @_;
+
+  unless (defined $path and $path ne '') {
+    croak "empty path for load_yatt!"
+  }
+
   $path = $self->rel2abs($path, $self->{cf_app_root});
   if (my $yatt = $self->{path2yatt}{$path}) {
     return $yatt;
@@ -568,14 +698,20 @@ sub build_yatt {
   $self->_list_base_spec_in($path, delete $opts{base}, $visits
 			    , \@basepkg, \@basevfs);
 
-  my $app_ns = $self->buildns(INST => \@basepkg, $path);
+  my $app_ns = $self->buildns(my @log = (INST => \@basepkg, $path));
 
-  if (-e (my $rc = "$path/.htyattrc.pl")) {
+  print STDERR "# Factory::buildns("
+    , terse_dump(@log), ") => $app_ns\n" if DEBUG_FACTORY;
+
+  my $has_rc;
+  if ($has_rc = (-e (my $rc = "$path/.htyattrc.pl"))) {
     # Note: This can do "use fields (...)"
     dofile_in($app_ns, $rc);
   }
 
-  my @args = (vfs => [dir => $path, encoding => $self->{cf_tmpl_encoding}
+  my @args = (vfs => [dir => $path
+		      , entns => $self->{path2entns}{$path}
+		      , encoding => $self->{cf_tmpl_encoding}
 		      , @basevfs ? (base => \@basevfs) : ()]
 	      , dir => $path
 	      , app_ns => $app_ns
@@ -585,7 +721,8 @@ sub build_yatt {
 	      # XXX: Design flaw! Use of tmpl_cache will cause problem.
 	      # because VFS->create for base do not respect Factory->get_yatt.
 	      # To solve this, I should redesign all Factory/VFS related stuffs.
-	      # , tmpl_cache => $self->{tmpl_cache} //= {}
+	      , tmpl_cache => $self->{tmpl_cache} //= {}
+	      , entns2vfs_item => $self->{entns2vfs_item} //= {}
 
 	      , $self->configparams_for(fields_hash($app_ns)));
 
@@ -594,51 +731,96 @@ sub build_yatt {
 		 , $path, join(", ", @unk));
   }
 
-  $self->{path2yatt}{$path} = $app_ns->new(@args, %opts);
+  my $yatt = $self->{path2yatt}{$path} = $app_ns->new(@args, %opts);
+
+  if ($has_rc) {
+    $yatt->setup_rc_actions;
+  }
+
+  $yatt;
 }
 
 sub _list_base_spec_in {
   (my MY $self, my ($in, $desc, $visits, $basepkg, $basevfs)) = @_;
 
+  print STDERR "# Factory::list_base_in("
+    , terse_dump($in, $desc, $self->{cf_app_base}), ")\n" if DEBUG_FACTORY;
+
+  #
+  # YATT::Lite->base can be either specified explicitly
+  # or implicitly copied from YATT::Lite::Factory->app_base.
+  #
+  # Later case can lead circular inheritance for app_base itself.
+  # To avoid this, $is_implicit flag is used.
+  #
   my $is_implicit = not defined $desc;
 
   $desc //= $self->{cf_app_base};
 
+  #
+  # First item in base is treated *primary* base.
+  # Rest of them are treated mixin.
+  #
   my ($base, @mixin) = lexpand($desc)
     or return;
 
-  my @pkg_n_dir;
+  #
+  # This builds [$package => $path] pairs and separately store
+  # as primary and mixin.
+  #
+  my (@primary_pair, @mixin_pair);
   foreach my $task ([1, $base], [0, @mixin]) {
     my ($is_primary, @spec) = @$task;
     foreach my $basespec (@spec) {
       my ($pkg, $yatt);
       if ($basespec =~ /^::(.*)/) {
 	ckrequire($1);
-	push @pkg_n_dir, [$is_primary, $1, undef];
+	push @{$is_primary ? \@primary_pair : \@mixin_pair}, [$1, undef];
       } elsif (my $realpath = $self->app_path_find_dir_in($in, $basespec)) {
+
 	if ($is_implicit) {
+	  #
+	  # Simply drop circular inheritance for implicit case.
+	  #
 	  next if $visits->has_node($realpath);
 	}
 	$visits->ensure_make_node($realpath);
-	push @pkg_n_dir, [$is_primary, undef, $realpath];
+
+	push @{$is_primary ? \@primary_pair : \@mixin_pair}, [undef, $realpath];
       } else {
 	$self->error("Invalid base spec: %s", $basespec);
       }
     }
   }
 
-  foreach my $tuple (@pkg_n_dir) {
-    my ($is_primary, $pkg, $dir) = @$tuple;
+  #
+  # This builds $basevfs for YATT::Lite::VFS::Folder::vivify_base_descs()
+  # This preallocates YATT::Lite and its entns for each realpath.
+  #
+  foreach my $pair (@primary_pair, @mixin_pair) {
+    my ($pkg, $dir) = @$pair;
     next unless $dir;
     my $yatt = $self->load_yatt($dir, undef, $visits, $in);
-    $tuple->[1] = ref $yatt;
-    push @$basevfs, [dir => $yatt->cget('dir')];
+    $pair->[0] = ref $yatt;
+    my $realdir = $yatt->cget('dir');
+    push @$basevfs, [dir => $realdir, entns => $self->{path2entns}{$realdir}];
   }
 
-  push @$basepkg, map {
-    my ($is_primary, $pkg, $dir) = @$_;
-    ($is_primary && $pkg) ? ($pkg) : ()
-  } @pkg_n_dir;
+  #
+  # This builds $basepkg for buildns()
+  #
+  push @$basepkg, map {defined $_->[0] ? $_->[0] : ()} do {
+    if (not $self->{cf_no_mro_c3}) {
+      my %known_pkg;
+      foreach my $pair (grep {defined $_->[0]} @primary_pair) {
+	$known_pkg{$_} = 1 for @{mro::get_linear_isa($pair->[0])};
+      }
+      (grep(!$_->[0] || !$known_pkg{$_->[0]}, @mixin_pair)
+       , @primary_pair);
+    } else {
+      @primary_pair;
+    }
+  };
 
   $visits->finish_node($in);
 }
@@ -666,6 +848,9 @@ sub buildns {
 			   , [])}, $self->EntNS;
   }
 
+  # basevfs に entns を渡せるように。
+  $self->{path2entns}{$path} = $newns->EntNS;
+
   $newns;
 }
 
@@ -684,6 +869,18 @@ sub _cf_delegates {
      ext_private
      only_parse
      use_subpath
+     dont_map_args
+     dont_debug_param
+     always_refresh_deps
+     no_mro_c3
+     die_in_error
+     special_entities
+     default_lang
+     no_lineinfo
+     ext_pattern
+     debug_parser
+     check_lineno
+     match_argsroute_first
   );
 }
 
