@@ -28,6 +28,8 @@ use fields qw/re_decl
 	      subroutes
 	      rootroute
 
+              cf_match_argsroute_first
+
 	      _original_entpath
 	    /;
 
@@ -312,8 +314,10 @@ sub parse_decl {
 
 sub finalize_template {
   (my MY $self, my Template $tmpl) = @_;
-  if ($self->{rootroute}) {
-    $self->subroutes->append($self->{rootroute});
+  if ($self->{cf_match_argsroute_first}) {
+    if ($self->{rootroute}) {
+      $self->subroutes->append($self->{rootroute});
+    }
   }
   if ($self->{subroutes}) {
     $tmpl->{cf_subroutes} = $self->{subroutes};
@@ -457,6 +461,7 @@ sub build {
   (my MY $self, my ($ns, $kind, $partName)) = splice @_, 0, 4;
   $self->can("build_$kind")->
     ($self, name => $partName, kind => $kind
+     , folder => $self->{template}
      , startpos => $self->{startpos}, @_);
 }
 # 今度はこっちが今一ね。
@@ -464,7 +469,11 @@ sub build_widget { shift->Widget->new(@_) }
 sub build_page { shift->Page->new(@_) }
 sub build_action {
   (my MY $self, my (%opts)) = @_;
+
+  # XXX: This is a workaround to have renderers and actions
+  # in the same hash table $template->{Item}.
   $opts{name} = "do_$opts{name}";
+
   $self->Action->new(%opts);
 }
 sub build_data { shift->Data->new(@_) }
@@ -473,12 +482,9 @@ sub build_data { shift->Data->new(@_) }
 # declare
 sub declare_base {
   (my MY $self, my Template $tmpl, my ($ns, @args)) = @_;
-  my $att = YATT::Lite::Constants::cut_first_att(\@args) or do {
-    die $self->synerror_at($self->{startln}, q{No base arg});
-  };
-  # !yatt:base dir="..."
-  #            PATH BODY
-  push @{$tmpl->{cf_base}}, [@$att[NODE_PATH, NODE_BODY]]; # XXX: 定形？
+
+  $self->{cf_vfs}->declare_base($self, $tmpl, $ns, @args);
+
   undef;
 }
 
@@ -512,7 +518,11 @@ sub declare_args {
 			       , q{Invalid location in %s:%s - "%s"}
 			       , $ns, 'args', $patNode->[NODE_BODY])
       };
-    $self->{rootroute} = $mapping;
+    if ($self->{cf_match_argsroute_first}) {
+      $self->{rootroute} = $mapping;
+    } else {
+      $self->{subroutes}->append($mapping);
+    }
     $self->add_url_params($newpart, lexpand($mapping->cget('params')));
   }
 
@@ -581,8 +591,6 @@ sub add_part {
   if (defined $tmpl->{Item}{$part->{cf_name}}) {
     die $self->synerror_at($self->{startln}, q{Conflicting part name! '%s'}, $part->{cf_name});
   }
-  Scalar::Util::weaken($part->{cf_folder} = $tmpl);
-  # die "Can't weaken!" unless Scalar::Util::isweak($part->{cf_folder});
   if ($tmpl->{partlist} and my Part $prev = $tmpl->{partlist}[-1]) {
     $prev->{cf_endln} = $self->{endln};
   }
@@ -603,9 +611,29 @@ sub add_lineinfo {
   # push @$sink, [TYPE_LINEINFO, $self->{endln}];
 }
 
+sub parse_arg_spec_for_part {
+  (my MY $self, my Part $part, my $attNode) = @_;
+    my ($node_type, $lno, $argName, $desc, @rest)
+      = @{$attNode}[NODE_TYPE, NODE_LNO, NODE_PATH, NODE_BODY
+		    , NODE_BODY+1 .. $#$attNode];
+  my ($type, $dflag, $default);
+  if ($node_type == TYPE_ATT_NESTED) {
+    $type = $desc->[NODE_PATH] || $desc->[NODE_BODY];
+    # primary of [primary key=val key=val] # delegate:foo の時は BODY に入る？
+  } else {
+    ($type, $dflag, $default) = split m{([|/?!])}, $desc || '', 2;
+  };
+  ($type, $argName, nextArgNo($part)
+   , $lno, $node_type, $dflag
+   , defined $default
+   ? $self->_parse_text_entities($default) : undef);
+}
+
 sub add_args {
   (my MY $self, my Part $part) = splice @_, 0, 2;
   foreach my $argSpec (@_) {
+    # XXX: Rewrite this with parse_arg_spec_for_part!
+
     # XXX: text もあるし、 %yatt:argmacro; もある。
     my ($node_type, $lno, $argName, $desc, @rest)
       = @{$argSpec}[NODE_TYPE, NODE_LNO, NODE_PATH, NODE_BODY
@@ -613,10 +641,7 @@ sub add_args {
     unless (defined $argName) {
       die $self->synerror_at($self->{startln}, 'Invalid argument spec');
     }
-    if (exists $part->{arg_dict}{$argName}) {
-      die $self->synerror_at($self->{startln}, 'Argument %s redefined in %s %s'
-		   , $argName, $part->{cf_kind}, $part->{cf_name});
-    }
+
     my ($type, $dflag, $default);
     if ($node_type == TYPE_ATT_NESTED) {
       $type = $desc->[NODE_PATH] || $desc->[NODE_BODY];
@@ -624,23 +649,38 @@ sub add_args {
     } else {
       ($type, $dflag, $default) = split m{([|/?!])}, $desc || '', 2;
     };
-    my $var = $self->mkvar_at($self->{startln}
-			      , $type, $argName, nextArgNo($part)
-			      , $lno, $node_type, $dflag
-			      , defined $default
-			      ? $self->_parse_text_entities($default) : undef);
 
-    if ($node_type == TYPE_ATT_NESTED) {
-      # XXX: [delegate:type ...], [code  ...] の ... が来る
-      # 仮想的な widget にする？ のが一番楽そうではあるか。そうすれば add_args 出来る。
-      # $self->add_arg_of_delegate/code/...へ。
-      my $t = $var->type->[0];
-      my $sub = $self->can("add_arg_of_type_$t")
-	or die $self->synerror_at($self->{startln}, "Unknown arg type in arg '%s': %s", $argName, $t);
-      $sub->($self, $part, $var, \@rest);
+    if (my $var = $part->{arg_dict}{$argName}) {
+      if ($var->from_route) {
+        # Override $type, $dflag, $default of this var.
+        $self->set_var_type($var, $type); # type is always overridden.
+        $var->dflag($dflag) if $dflag;
+        $var->default($self->_parse_text_entities($default))
+          if defined $default;
+      } else {
+        die $self->synerror_at($self->{startln}
+                               , 'Argument %s redefined in %s %s'
+                               , $argName, $part->{cf_kind}, $part->{cf_name});
+      }
     } else {
-      push @{$part->{arg_order}}, $argName;
-      $part->{arg_dict}{$argName} = $var;
+      my $var = $self->mkvar_at($self->{startln}
+                                , $type, $argName, nextArgNo($part)
+                                , $lno, $node_type, $dflag
+                                , defined $default
+                                ? $self->_parse_text_entities($default) : undef);
+
+      if ($node_type == TYPE_ATT_NESTED) {
+        # XXX: [delegate:type ...], [code  ...] の ... が来る
+        # 仮想的な widget にする？ のが一番楽そうではあるか。そうすれば add_args 出来る。
+        # $self->add_arg_of_delegate/code/...へ。
+        my $t = $var->type->[0];
+        my $sub = $self->can("add_arg_of_type_$t")
+          or die $self->synerror_at($self->{startln}, "Unknown arg type in arg '%s': %s", $argName, $t);
+        $sub->($self, $part, $var, \@rest);
+      } else {
+        push @{$part->{arg_order}}, $argName;
+        $part->{arg_dict}{$argName} = $var;
+      }
     }
   }
   $self;
@@ -653,12 +693,12 @@ sub add_url_params {
     my $type = 'value'; # XXX: type_or_pat
     my $var = $self->mkvar_at($self->{startln}, $type, $argName
 			      , nextArgNo($part));
+    $var->from_route(1);
     push @{$part->{arg_order}}, $argName;
     $part->{arg_dict}{$argName} = $var;
   }
 }
 
-
 # code 型は仮想的な Widget を作る。
 sub add_arg_of_type_code {
   (my MY $self, my Part $part, my ($var, $attlist)) = @_;
@@ -676,21 +716,66 @@ sub add_arg_of_type_delegate {
   # XXX: 既に有ったらエラーにしないと。
   $widget->{var_dict}{$name} = $var;
   my ($type, @subtype) = @{$var->type};
+  my @wpath = @subtype ? @subtype : $name;
   my Widget $delegate = $self->{cf_vfs}->find_part_from
-    ($widget->{cf_folder}, @subtype ? @subtype : $name);
+    ($widget->{cf_folder}, @wpath) or do {
+      $self->synerror_at($self->{startln}, "Can't find delegate widget for argument %s=[%s]", $name, join(":", $type, @subtype));
+    };
   $var->weakened_set_widget($delegate);
   unless (Scalar::Util::isweak($var->[YATT::Lite::VarTypes::t_delegate::VSLOT_WIDGET])) {
     die "Can't weaken!";
   }
   $var->delegate_vars(\ my %delegate_vars);
+
+  my ($attDict, $excludeDict) = do {
+    my (%attDict, %exclDict);
+    foreach my $argSpec (@$attlist) {
+      if (my $attName = $argSpec->[NODE_PATH]) {
+	defined $attDict{$attName}
+	  and die $self->synerror_at
+	  ($argSpec->[NODE_LNO]
+	   , "Duplicate argname '%s' in delegate var %s"
+	   , $attName, $name);
+	$attDict{$attName} = $argSpec;
+      } elsif ($argSpec->[NODE_TYPE] == TYPE_ATT_TEXT
+	       and ($attName) = $argSpec->[NODE_BODY] =~ /^-(\w+)$/) {
+	if (not $delegate->{arg_dict}{$attName}) {
+	  die $self->synerror_at
+	    ($argSpec->[NODE_LNO]
+	     , "No such argument '%s' in delegate to '%s'"
+	     , $attName, $name);
+	}
+	$exclDict{$attName} = $argSpec;
+      } else {
+	die $self->synerror_at
+	  ($argSpec->[NODE_LNO]
+	   , "Invalid decl spec for delegate var %s", $name);
+      }
+    }
+    (\%attDict, \%exclDict);
+  };
+
   foreach my $argName (@{$delegate->{arg_order}}) {
     # 既に宣言されている名前は、足さない。
     next if $widget->{arg_dict}{$argName};
+
+    # Ignore [delegate -excluded_var]
+    next if $excludeDict->{$argName};
+
     $delegate_vars{$argName} = my $orig = $delegate->{arg_dict}{$argName};
-    # clone して argno と lineno を変える。
-    $widget->{arg_dict}{$argName} = my $clone
-      = $self->mkvar_at($widget->{cf_startln}, @$orig)
-	->argno(nextArgNo($widget))->lineno($widget->{cf_startln});
+
+    my $actual = do {
+      if (my $att = $attDict->{$argName}) {
+	my @new = $self->parse_arg_spec_for_part($widget, $att);
+	$new[0] ||= $orig->[0];
+	$self->mkvar_at($self->{startln}, @new);
+      } else {
+	# clone して argno と lineno を変える。
+	$self->mkvar_at($widget->{cf_startln}, @$orig)
+	  ->argno(nextArgNo($widget))->lineno($widget->{cf_startln});
+      }
+    };
+    $widget->{arg_dict}{$argName} = $actual;
     # XXX: lineno を widget の startln にするのは手抜き。本来は直前の arg のものを使うべき。
     push @{$widget->{arg_order}}, $argName;
   }
