@@ -17,6 +17,8 @@ BEGIN {
 	  PATH_INFO
 	  PATH_TRANSLATED
 	  REDIRECT_STATUS
+          REDIRECT_HANDLER
+          REDIRECT_URL
 	  REQUEST_URI
 	  DOCUMENT_URI
 
@@ -58,6 +60,8 @@ BEGIN {
 	  psgix.session
 	  psgix.session.options
 	  psgix.logger
+
+          yatt.script_name
        /;
   our %FIELDS = map {$_ => ''} @PSGI_FIELDS;
 }
