@@ -1,6 +1,6 @@
 package YATT::Lite::RegexpNames;
 use strict;
-use warnings FATAL => qw/all/;
+use warnings qw(FATAL all NONFATAL misc);
 
 use Exporter qw/import/;
 use YATT::Lite::Util qw/globref symtab/;
