@@ -1,6 +1,6 @@
 package YATT::Lite::Test::XHFTest;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use parent qw(YATT::Lite::Object);
 use YATT::Lite::MFields qw/tests numtests yatt global file_list file_dict
 	      cf_filename cf_ext cf_parser cf_encoding
@@ -191,22 +191,6 @@ sub as_vfs_data {
   \%result;
 }
 
-sub as_real_files {
-  (my MY $self, my $basedir, my @opts) = @_;
-  require YATT::TestFiles;
-  my $builder = $self->{builder} = new YATT::TestFiles($basedir, @opts);
-  foreach my $fn (@{$self->{file_list}}) {
-    my Item $item = $self->{file_dict}{$fn};
-    $builder->add($fn, $item->{cf_IN});
-  }
-}
-
-sub convert_htyattrc {
-  (my MY $self, my $path_cursor, my Item $item) = @_;
-  # XXX: 未実装
-  # data =>, rc => '...'
-}
-
 sub path_cursor {
   my ($top, $path) = @_;
   # path を一個残して、vivify する。
