@@ -1,6 +1,9 @@
 package YATT::Lite::Util;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
+use constant DEBUG_LOOKUP_PATH => $ENV{DEBUG_YATT_UTIL_LOOKUP_PATH};
+
+use Encode ();
 
 use URI::Escape ();
 use Tie::IxHash;
@@ -16,6 +19,7 @@ require Scalar::Util;
 		     untaint_any ckeval ckrequire untaint_unless_tainted
 		     dict_sort terse_dump catch
 		     nonempty
+		     empty
 		     subname
 		     pkg2pm
 		     globref_default
@@ -37,6 +41,9 @@ require Scalar::Util;
   sub nonempty {
     defined $_[0] && $_[0] ne '';
   }
+  sub empty {
+    not defined $_[0] or $_[0] eq '';
+  }
 
   sub define_const {
     my ($name_or_glob, $value) = @_;
@@ -77,6 +84,7 @@ require Scalar::Util;
       or return undef;
     *{$sym}{HASH};
   }
+  # XXX: should be renamed to lhexpand
   sub lexpand {
     # lexpand can be used to counting.
     unless (defined $_[0]) {
@@ -169,10 +177,11 @@ require Scalar::Util;
   }
 
   sub split_path {
-    my ($path, $startDir, $cut_depth) = @_;
+    my ($path, $startDir, $cut_depth, $default_ext) = @_;
     # $startDir is $app_root.
     # $doc_root should resides under $app_root.
     $cut_depth //= 1;
+    $default_ext //= "yatt";
     $startDir =~ s,/+$,,;
     unless ($path =~ m{^\Q$startDir\E}gxs) {
       die "Can't split_path: prefix mismatch: $startDir vs $path";
@@ -194,10 +203,10 @@ require Scalar::Util;
     $dir .= "/" if $dir !~ m{/$};
     my $subpath = substr($path, $pos);
     if (not defined $file) {
-      if ($subpath =~ m{^/(\w+)(?:/|$)} and -e "$dir/$1.yatt") {
+      if ($subpath =~ m{^/(\w+)(?:/|$)} and -e "$dir/$1.$default_ext") {
 	$subpath = substr($subpath, 1+length $1);
-	$file = "$1.yatt";
-      } elsif (-e "$dir/index.yatt") {
+	$file = "$1.$default_ext";
+      } elsif (-e "$dir/index.$default_ext") {
 	# index.yatt should subsume all subpath.
       } elsif ($subpath =~ s{^/([^/]+)$}{}) {
 	# Note: Actually, $file is not accesible in this case.
@@ -237,12 +246,14 @@ require Scalar::Util;
     $want_ext //= '.yatt';
     my $ixfn = $index_name . $want_ext;
     my @dirlist = grep {defined $_ and -d $_} @$dirlist;
+    print STDERR "dirlist" => terse_dump(@dirlist), "\n" if DEBUG_LOOKUP_PATH;
     my $pi = $path_info;
     my ($loc, $cur, $ext) = ("", "");
   DIG:
     while ($pi =~ s{^/+([^/]+)}{}) {
       $cur = $1;
       $ext = ($cur =~ s/(\.[^\.]+)$// ? $1 : undef);
+      print STDERR terse_dump(cur => $cur, ext => $ext), "\n" if DEBUG_LOOKUP_PATH;
       foreach my $dir (@dirlist) {
 	my $base = "$dir$loc/$cur";
 	if (defined $ext and -r "$base$ext") {
@@ -264,8 +275,10 @@ require Scalar::Util;
       }
     } continue {
       $loc .= "/$cur";
+      print STDERR terse_dump(continuing => $loc), "\n" if DEBUG_LOOKUP_PATH;
       @dirlist = grep {defined} @dirlist;
     }
+      print STDERR terse_dump('end_of_loop'), "\n" if DEBUG_LOOKUP_PATH;
 
     return unless $pi =~ m{^/+$};
 
@@ -274,9 +287,27 @@ require Scalar::Util;
       return ($dir, "$loc/", "$ixfn", "", 1);
     }
 
+      print STDERR terse_dump('at_last'), "\n" if DEBUG_LOOKUP_PATH;
     return;
   }
 
+  sub trim_common_suffix_from {
+    @_ == 2 or Carp::croak "trim_common_suffix_from(FROM, COMPARE)";
+    my $from = $_[0];
+    my $cmppos = (length $_[1]) - 1;
+    while (length $from and $cmppos >= 0) {
+      (my $slash = rindex($_[1], "/", $cmppos)) >= 0
+        or last;
+      (my $pos = rindex($from, substr($_[1], $slash, $cmppos - $slash))) >= 0
+        or last;
+
+      substr($from, $pos) = "";
+
+      $cmppos = $slash - 1;
+    }
+    $from;
+  }
+
   sub dict_order {
     my ($a, $b, $start) = @_;
     $start = 1 unless defined $start;
@@ -298,10 +329,30 @@ require Scalar::Util;
   sub dict_sort (@) {
     map {$_->[0]} sort {dict_order($a,$b)} map {[$_, split /(\d+)/]} @_;
   }
+  sub dict_sort_by_nth ($@) {
+    my $nth = shift;
+    map {$_->[0]}
+    sort {dict_order($a,$b)}
+    map {[$_, split /(\d+)/, $$_[$nth]]} @_;
+  }
+
+  sub combination (@) {
+    my $comb; $comb = sub {
+      my $prefix = shift;
+      return $prefix unless @_;
+      my ($list, @rest) = @_;
+      if (@rest) {
+        map {$comb->([@$prefix, $_], @rest)} @$list;
+      } else {
+        map {[@$prefix, $_]} @$list;
+      }
+    };
+    $comb->([], @_);
+  }
 
   sub captured (&) {
     my ($code) = @_;
-    open my $fh, '>', \ (my $buffer = "") or die "Can't create capture buf:$!";
+    open my $fh, '>:utf8', \ (my $buffer = "") or die "Can't create capture buf:$!";
     $code->($fh);
     $buffer;
   }
@@ -338,6 +389,9 @@ sub dofile_in {
 
 sub compile_file_in {
   my ($pkg, $file) = @_;
+  if (-d $file) {
+    croak "file '$file' is a directory!";
+  }
   my $sub = dofile_in($pkg, $file);
   unless (defined $sub and ref $sub eq 'CODE') {
     die "file '$file' should return CODE (but not)!\n";
@@ -369,7 +423,7 @@ BEGIN {
 	  $copy;
 	} elsif (ref $str eq 'SCALAR') {
 	  # PASS Thru. (Already escaped)
-	  $$str;
+	  $$str // $ESCAPE_UNDEF; # fail safe
 	} elsif (_is_escapable($str)) {
 	  $str->as_escaped;
 	} elsif (my $sub = UNIVERSAL::can($str, 'cf_pairs')) {
@@ -460,7 +514,7 @@ sub named_attr {
 		     , radio => [1, 0]
 		     , checkbox => [2, 1]);
   sub att_value_in {
-    my ($in, $type, $name, $formal_value, $as_value) = @_;
+    my ($in, $type, $name, $formal_value, $as_value, $is_default) = @_;
     defined (my $spec = $input_spec{$type})
       or croak "Unknown type: $type";
 
@@ -493,7 +547,7 @@ sub named_attr {
       push @res, qq|value="@{[escape($as_value)]}"|;
     }
 
-    if (find_value_in($in, $name, $formal_value)) {
+    if (find_value_in($in, $name, $formal_value, $is_default)) {
       push @res, $typeid ? "checked" : "selected";
     }
 
@@ -501,7 +555,7 @@ sub named_attr {
   }
 
   sub find_value_in {
-    my ($in, $name, $formal_value) = @_;
+    my ($in, $name, $formal_value, $is_default) = @_;
 
     my $actual_value = do {
       if (my $sub = $in->can("param")) {
@@ -509,12 +563,12 @@ sub named_attr {
       } elsif (ref $in eq 'HASH') {
 	$in->{$name};
       } else {
-	croak "Can't extract parameter from $in";
+	undef;
       }
     };
 
     if (not defined $actual_value) {
-      0
+      $is_default ? 1 : 0
     } elsif (not ref $actual_value) {
       $actual_value eq $formal_value
     } elsif (ref $actual_value eq 'HASH') {
@@ -528,17 +582,24 @@ sub named_attr {
 }
 
 # Verbatimly stolen from CGI::Simple
+# XXX: not used?
 sub url_decode {
   my ( $self, $decode ) = @_;
   return () unless defined $decode;
   $decode =~ tr/+/ /;
   $decode =~ s/%([a-fA-F0-9]{2})/ pack "C", hex $1 /eg;
+  # XXX: should set utf8 flag too?
   return $decode;
 }
 
 sub url_encode {
   my ( $self, $encode ) = @_;
   return () unless defined $encode;
+
+  if (Encode::is_utf8($encode)) {
+    $encode = Encode::encode_utf8($encode);
+  }
+
   # XXX: Forward slash (and ':') is allowed, for cleaner url. This may break...
   $encode
     =~ s{([^A-Za-z0-9\-_.!~*'() /:])}{ uc sprintf "%%%02x",ord $1 }eg;
@@ -568,7 +629,7 @@ sub encode_query {
       my @res;
       while (my ($k, $v) = splice @param, 0, 2) {
 	my $ek = url_encode($self, $k);
-	push @res, $ek . '='. url_encode($self, $_)
+	push @res, $ek . '='. (url_encode($self, $_) // '')
 	  for ref $v ? @$v : $v;
       }
       @res;
@@ -662,12 +723,6 @@ sub list_isa {
   } @$isa;
 }
 
-sub get_locale_encoding {
-  require Encode;
-  require encoding;
-  Encode::find_encoding(encoding::_get_locale_encoding())->name;
-}
-
 sub set_inc {
   my ($pkg, $val) = @_;
   $pkg =~ s|::|/|g;
@@ -680,7 +735,8 @@ sub try_invoke {
   my $obj = shift;
   my ($method, @args) = lexpand(shift);
   my $default = shift;
-  if (my $sub = UNIVERSAL::can($obj, $method)) {
+  if (defined $obj
+      and my $sub = UNIVERSAL::can($obj, $method)) {
     $sub->($obj, @args);
   } else {
     wantarray ? () : $default;
@@ -759,13 +815,21 @@ sub parse_nested_query {
   return {} unless defined $_[0] and $_[0] ne '';
   my ($enc) = $_[1];
   my $params = $_[2] // ixhash();
-  foreach my $p (split /[;&]/, $_[0]) {
-    my ($k, $v) = map {
-      s/\+/ /g;
-      my $raw = URI::Escape::uri_unescape($_);
-      $enc ? Encode::decode($enc, $raw) : $raw;
-    } split /=/, $p, 2;
-    normalize_params($params, $k, $v) if defined $k;
+  if (ref $_[0]) {
+    my @pairs = map {$enc ? map(Encode::decode($enc, $_), @$_) : @$_}
+      ref $_[0] eq 'ARRAY' ? $_[0] : [%{$_[0]}];
+    while (my ($k, $v) = splice @pairs, 0, 2) {
+      normalize_params($params, $k, $v);
+    }
+  } else {
+    foreach my $p (split /[;&]/, $_[0]) {
+      my ($k, $v) = map {
+	s/\+/ /g;
+	my $raw = URI::Escape::uri_unescape($_);
+	$enc ? Encode::decode($enc, $raw) : $raw;
+      } split /=/, $p, 2;
+      normalize_params($params, $k, $v) if defined $k;
+    }
   }
   $params;
 }
