@@ -1,8 +1,8 @@
 package YATT::Lite::VarMaker; sub MY () {__PACKAGE__}
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use base qw(YATT::Lite::Object);
-use fields qw/type_alias
+use YATT::Lite::MFields qw/type_alias
 	      curline/;
 
 use YATT::Lite::VarTypes qw(:type :VSLOT);
@@ -29,20 +29,36 @@ sub after_new {
 # Currently, YATT::Lite::LRXML and YATT::Lite::CGen uses this.
 sub mkvar_at {
   (my MY $self, my ($lineno, $type, $name, @args)) = @_;
+
+  my ($typerec, $sub) = $self->_mk_typerec($type, $lineno, $name);
+
+  my $var = $sub->()->new($typerec, $name, @args);
+  $var->[VSLOT_LINENO] //= $lineno //= $self->{curline};
+  $var;
+}
+
+sub set_var_type {
+  (my MY $self, my ($var, $type)) = @_;
+  $var->type(scalar $self->_mk_typerec($type));
+}
+
+sub _mk_typerec {
+  (my MY $self, my ($type, $lineno, $name)) = @_;
+
   ($type, my @subtype) = ref $type ? lexpand($type) : split /:/, $type || '';
   #
   $type ||= $self->default_arg_type;
   $type = default($self->{type_alias}{$type}, $type);
 
-  my $sub = $self->can("t_$type") or do {
-    my %opts = ($self->_tmpl_file_line($lineno));
-    die $self->_error(\%opts, q|Unknown type '%s' for variable '%s'|
-		      , $type, $name);
-  };
+  my $typerec = [$type, @subtype];
 
-  my $var = $sub->()->new([$type, @subtype], $name, @args);
-  $var->[VSLOT_LINENO] //= $lineno //= $self->{curline};
-  $var;
+  wantarray ? ($typerec, do {
+    my $sub = $self->can("t_$type") or do {
+      my %opts = ($self->_tmpl_file_line($lineno));
+      die $self->_error(\%opts, q|Unknown type '%s' for variable '%s'|
+                        , $type, $name);
+    };
+  }) : $typerec;
 }
 
 1;
