@@ -1,6 +1,6 @@
 package YATT::Lite::VarTypes; sub MY () {__PACKAGE__}
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Exporter qw(import);
 
 sub Base () {'YATT::Lite::VarTypes::Base'}
@@ -23,7 +23,9 @@ sub add_slot_to {
 BEGIN {
   our @fields = qw(type
 		   varname argno
-		   lineno quote dflag default);
+		   lineno quote dflag default
+                   from_route
+                );
   my $slotNum = 0;
   foreach my $name (@fields) {
     # accessor
