@@ -1,6 +1,6 @@
 package YATT::Lite::WebMVC0::DBSchema; sub MY () {__PACKAGE__}
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 use File::Basename;
 use version;
@@ -298,7 +298,7 @@ sub dbi_connect {
 }
 
 sub default_dbi_attr {
-  (RaiseError => 1, PrintError => 0, AutoCommit => 0);
+  (RaiseError => 1, PrintError => 0, AutoCommit => 1);
 }
 
 #----------------------------------------
@@ -382,7 +382,10 @@ sub ensure_table_populated {
 sub sqlite_begin_create {
   (my MY $schema) = @_;
   # To speedup create statements.
-  $schema->dbh->do("PRAGMA synchronous = OFF");
+  my $dbh = $schema->dbh;
+  if ($dbh->{AutoCommit}) {
+    $dbh->do("PRAGMA synchronous = OFF");
+  }
 }
 
 sub expand_codevalue {
