@@ -1,10 +1,14 @@
 package YATT::Lite::WebMVC0::DirApp; sub MY () {__PACKAGE__}
 use strict;
 use warnings qw(FATAL all NONFATAL misc);
+use Carp;
+use mro 'c3';
+
 use YATT::Lite -as_base, qw/*SYS
 			    Entity/;
 use YATT::Lite::MFields qw/cf_dir_config
 			   cf_use_subpath
+			   cf_overwrite_status_code_for_errors_as
 
 			   Action/;
 
@@ -12,7 +16,6 @@ use YATT::Lite::WebMVC0::Connection;
 sub Connection () {'YATT::Lite::WebMVC0::Connection'}
 sub PROP () {Connection}
 
-use Carp;
 use YATT::Lite::Util qw/cached_in ckeval
 			dofile_in compile_file_in
 			try_invoke
@@ -20,6 +23,8 @@ use YATT::Lite::Util qw/cached_in ckeval
 			terse_dump
 		      /;
 
+use YATT::Lite::Error;
+
 # sub handle_ydo, _do, _psgi...
 
 sub handle {
@@ -68,14 +73,16 @@ sub prepare_part_handler {
     ($part, my ($formal, $actual)) = $tmpl->match_subroutes($subpath) or do {
       # XXX: Is this secure against XSS? <- how about URI encoding?
       # die $self->psgi_error(404, "No such subpath: ". $subpath);
-      die $self->psgi_error(404, "No such subpath");
+      die $self->psgi_error(404, "No such subpath:: ". $subpath
+			    . " in file " . $tmpl->{cf_path});
     };
     $pkg = $trans->find_product(perl => $tmpl) or do {
       croak $self->error("Can't compile template file: %s", $file);
     };
-    my $name = $part->cget('name');
-    $sub = $pkg->can("render_$name") or do {
-      croak $self->error("Can't find page %s for file: %s", $name, $file);
+
+    $sub = $pkg->can($part->method_name) or do {
+      croak $self->error("Can't find %s %s for file: %s"
+                         , $part->cget('kind'), $part->public_name, $file);
     };
     @args = $part->reorder_cgi_params($con, $actual)
       unless $self->{cf_dont_map_args};
@@ -205,26 +212,47 @@ sub fn_msgfile {
 
 #========================================
 sub error_handler {
-  (my MY $self, my ($type, $err)) = @_;
+  (my MY $self, my $type, my Error $err) = @_;
   # どこに出力するか、って問題も有る。 $CON を rewind すべき？
-  my $errcon = do {
-    if (my $con = $self->CON) {
-      $con->as_error;
-    } elsif ($SYS) {
+  my $errcon = try_invoke($self->CON, 'as_error') || do {
+    if ($SYS) {
       $SYS->make_connection(\*STDOUT, yatt => $self, noheader => 1);
     } else {
       \*STDERR;
     }
   };
-  # error.ytmpl を探し、あれば呼び出す。
-  my ($sub, $pkg) = $self->find_renderer($type => ignore_error => 1) or do {
-    # print {*$errcon} $err, Carp::longmess(), "\n\n";
-    # Dispatcher の show_error に任せる
+  if (my $code = $err->{cf_http_status_code}) {
+    $errcon->configure(status => $code);
+  } elsif ($code = try_invoke($errcon, [cget => 'status'])) {
+    $err->{cf_http_status_code} = $code;
+  }
+
+  # yatt/ytmpl 用の Code generator がまだ無いので、素直に raise.
+  # XXX: 本当は正しくロードできる可能性もあるが,
+  #  そこで更に fail すると真のエラーが隠されてしまうため、頑張らない。
+  unless ($self->is_default_cgen_ready) {
     die $err;
+  }
+
+  # error.ytmpl を探し、あれば呼び出す。
+  my ($sub, $pkg);
+  ($sub, $pkg) = $self->find_renderer($type => ignore_error => 1) or do {
+    if ($err->{cf_http_status_code}
+	|| $self->{cf_overwrite_status_code_for_errors_as}) {
+       ($sub, $pkg) = (sub {
+			 my ($this, $errcon, $err) = @_;
+			 print {*$errcon} $err->reason;
+		       }, $self->EntNS);
+     } else {
+       die $err;
+     }
   };
   $sub->($pkg, $errcon, $err);
   try_invoke($errcon, 'flush_headers');
-  $self->raise_psgi_html(200, $errcon->buffer); # ->DONE was not ok.
+  $self->raise_psgi_html($self->{cf_overwrite_status_code_for_errors_as}
+			 // $errcon->cget('status')
+			 // 500
+			 , $errcon->buffer); # ->DONE was not ok.
 }
 
 Entity dir_config => sub {
