@@ -22,11 +22,19 @@ use YATT::Lite::MFields qw/cf_noheader
 			   cf_debug_backend
 			   cf_psgi_static
 			   cf_psgi_fallback
+			   cf_per_role_docroot
+			   cf_per_role_docroot_key
+			   cf_default_role
 			   cf_backend
 			   cf_site_config
 			   cf_logfile
 			   cf_debug_allowed_ip
+			   cf_overwrite_status_code_for_errors_as
 			   re_handled_ext
+
+                           cf_progname
+
+                           cf_no_trim_script_name
 			 /;
 
 use YATT::Lite::Util qw(cached_in split_path catch
@@ -34,6 +42,7 @@ use YATT::Lite::Util qw(cached_in split_path catch
 			mk_http_status
 			default ckrequire
 			escape
+                        trim_common_suffix_from
 			lexpand rootname extname untaint_any terse_dump);
 use YATT::Lite::Util::CmdLine qw(parse_params);
 use YATT::Lite qw/Entity *SYS *CON/;
@@ -47,6 +56,16 @@ sub after_new {
   (my MY $self) = @_;
   $self->SUPER::after_new();
   $self->{re_handled_ext} = qr{\.($self->{cf_ext_public}|ydo)$};
+  $self->{cf_per_role_docroot_key} ||= $self->default_per_role_docroot_key;
+  $self->{cf_default_role} ||= $self->default_default_role;
+}
+
+sub default_per_role_docroot_key { 'yatt.role' }
+sub default_default_role { 'nobody' }
+
+sub _cf_delegates {
+  (shift->SUPER::_cf_delegates
+   , qw(overwrite_status_code_for_errors_as));
 }
 
 #========================================
@@ -70,6 +89,26 @@ sub runas_fcgi {
   shift->_runas_fcgi(@_);
 }
 
+# callas($type, $app, $fh, \%ENV, \@ARGV, %opts)
+
+sub callas {
+  (my $this, my $type) = splice @_, 0, 2;
+  my MY $self = ref $this ? $this : $this->new;
+  my $sub = $self->can("callas_$type")
+    or die "\n\nUnknown callas type: $type";
+  $sub->($self, @_);
+}
+
+sub callas_cgi {
+  require YATT::Lite::WebMVC0::SiteApp::CGI;
+  shift->_callas_cgi(@_);
+}
+sub callas_fcgi {
+  require YATT::Lite::WebMVC0::SiteApp::FCGI;
+  shift->_callas_fcgi(@_);
+}
+
+
 #========================================
 
 # Dispatcher::get_dirhandler
@@ -77,10 +116,15 @@ sub runas_fcgi {
 # -> Factory::load
 # -> Factory::buildspec
 
-sub get_dirhandler {
-  (my MY $self, my $dirPath) = @_;
-  $dirPath =~ s,/*$,,;
-  $self->{path2yatt}{$dirPath} ||= $self->load_yatt($dirPath);
+sub get_lochandler {
+  (my MY $self, my ($location, $tmpldir)) = @_;
+  if ($self->{cf_per_role_docroot}) {
+    # When per_role_docroot is on, $tmpldir already points
+    # $per_role_docroot/$role. So just append $location.
+    $self->get_dirhandler($tmpldir.$location);
+  } else {
+    $self->SUPER::get_lochandler($location, $tmpldir);
+  }
 }
 
 #----------------------------------------
@@ -123,7 +167,8 @@ sub to_app {
 #  unless (defined $self->{cf_app_root}) {
 #    croak "app_root is undef!";
 #  }
-  unless (defined $self->{cf_doc_root}) {
+  unless (defined $self->{cf_doc_root}
+	  or defined $self->{cf_per_role_docroot}) {
     croak "document_root is undef!";
   }
   return $self->SUPER::to_app(@_);
@@ -172,11 +217,23 @@ sub call {
   my ($tmpldir, $loc, $file, $trailer, $is_index)
     = my @pi = $self->split_path_info($env);
 
+  # Set $env->{yatt.script_name}
+  $self->set_yatt_script_name($env);
+
+  my ($realdir, $virtdir);
+  if (@pi) {
+    $realdir = "$tmpldir$loc";
+    $virtdir = defined $self->{cf_doc_root}
+      ? "$self->{cf_doc_root}$loc" : $realdir;
+  }
+
   if ($self->has_htdebug("path_info")) {
     return $self->psgi_dump([tmpldir   => $tmpldir]
 			    , [loc     => $loc]
 			    , [file    => $file]
-			    , [trailer => $trailer]);
+			    , [trailer => $trailer]
+			    , [virtdir => $virtdir, realdir => $realdir]
+			  );
   }
 
   if ($self->{cf_debug_psgi}) {
@@ -198,10 +255,8 @@ sub call {
     return $self->psgi_handle_fallback($env);
   }
 
-  my $virtdir = "$self->{cf_doc_root}$loc";
-  my $realdir = "$tmpldir$loc";
   unless (-d $realdir) {
-    return $self->psgi_error(404, "Not found: $virtdir");
+    return $self->psgi_error(404, "Not found: $loc");
   }
 
   # Default index file.
@@ -252,11 +307,11 @@ sub call {
     }
     $res->body($con->buffer);
     return $res->finalize;
-  } elsif (ref $error eq 'ARRAY') {
+  } elsif (ref $error eq 'ARRAY' or ref $error eq 'CODE') {
     # redirect
     if ($self->{cf_debug_psgi}) {
       if (my $errfh = fileno(STDERR) ? \*STDERR : $env->{'psgi.errors'}) {
-	print $errfh "PSGI Tuple: ", terse_dump($error), "\n";
+	print $errfh "PSGI Response: ", terse_dump($error), "\n";
       }
     }
     return $error;
@@ -291,12 +346,6 @@ sub make_debug_params {
 
 #========================================
 
-sub psgi_file_app {
-  my ($pack, $path) = @_;
-  require Plack::App::File;
-  Plack::App::File->new(root => $path)->to_app;
-}
-
 sub psgi_handle_static {
   (my MY $self, my Env $env) = @_;
   my $app = $self->{cf_psgi_static}
@@ -311,8 +360,9 @@ sub psgi_handle_static {
 
 sub psgi_handle_fallback {
   (my MY $self, my Env $env) = @_;
-  my $app = $self->{cf_psgi_fallback}
-    or return [404, [], ["Cannot understand:", $env->{PATH_INFO}]];
+  (my $app = $self->{cf_psgi_fallback}
+   ||= $self->psgi_file_app($self->{cf_doc_root}))
+    or return [404, [], ["Cannot understand: ", $env->{PATH_INFO}]];
 
   local $env->{PATH_INFO} = $self->trim_site_prefix($env->{PATH_INFO});
 
@@ -341,10 +391,34 @@ sub is_done {
 
 #========================================
 
+sub set_yatt_script_name {
+  (my MY $self, my Env $env) = @_;
+
+  $env->{'yatt.script_name'} = do {
+    if (not $self->{cf_no_trim_script_name}
+        and $env->{REDIRECT_HANDLER}
+        and ($env->{REDIRECT_STATUS} // 0) == 200
+        and $env->{SCRIPT_FILENAME}
+      ) {
+      #
+      # For Apache Action+AddHandler mapping.
+      #
+      trim_common_suffix_from($env->{SCRIPT_NAME}
+                              , $env->{SCRIPT_FILENAME});
+    } else {
+      #
+      # Normal case.
+      #
+      $env->{SCRIPT_NAME};
+    }
+  };
+}
+
 sub split_path_info {
   (my MY $self, my Env $env) = @_;
 
-  if (nonempty($env->{PATH_TRANSLATED})
+  if (! $self->{cf_per_role_docroot}
+      && nonempty($env->{PATH_TRANSLATED})
       && $self->is_path_translated_mode($env)) {
     #
     # [1] PATH_TRANSLATED mode.
@@ -359,18 +433,40 @@ sub split_path_info {
     # XXX: Is userdir ok? like /~$USER/dir?
     # XXX: should have cut_depth option.
     #
-    split_path($env->{PATH_TRANSLATED}, $self->{cf_app_root}
-	       , $self->{cf_use_subpath}
-	       , $self->{cf_ext_public}
-	     );
-    # or die.
+    my ($tmpldir, $loc, $file, $trailer, $is_index)
+      = split_path($env->{PATH_TRANSLATED}, $self->{cf_app_root}
+                   , $self->{cf_use_subpath}
+                   , $self->{cf_ext_public}
+                 );
+
+    # This is a workaround for $is_index. Determining $is_index only from
+    # PATH_TRANSLATED was just wrong.
+    #
+    # So instead turn on it when "$env->{REQUEST_URI}$file" eq $env->{REDIRECT_URL};
+    #
+    $is_index ||= ($env->{REQUEST_URI}
+                   and $env->{REDIRECT_URL}
+                   and "$env->{REQUEST_URI}$file" eq $env->{REDIRECT_URL});
+
+    ($tmpldir, $loc, $file, $trailer, $is_index);
 
   } elsif (nonempty($env->{PATH_INFO})) {
     #
     # [2] Template lookup mode.
     #
+
+    my $tmpldirs = do {
+      if ($self->{cf_per_role_docroot}) {
+        my $user = $env->{$self->{cf_per_role_docroot_key}};
+        $user ||= $self->{cf_default_role};
+        ["$self->{cf_per_role_docroot}/$user"]
+      } else {
+        $self->{tmpldirs}
+      }
+    };
+
     lookup_path($env->{PATH_INFO}
-		, $self->{tmpldirs}
+		, $tmpldirs
 		, $self->{cf_index_name}, ".$self->{cf_ext_public}"
 		, $self->{cf_use_subpath});
   } else {
@@ -521,7 +617,9 @@ sub make_connection {
   }
   $self->SUPER::make_connection
   (@opts
-   , $self->cf_delegate_defined(qw(is_psgi no_nested_query))
+   , $self->cf_delegate_defined(qw(is_psgi
+                                   no_unicode_params
+                                   no_nested_query))
    , @args);
 }
 
@@ -571,6 +669,46 @@ Entity is_debug_allowed_ip => sub {
 				    // ['127.0.0.1']);
 };
 
+foreach my $name (qw/
+		      file_location
+		      dir_location
+		      is_current_file
+		      is_current_page
+		    /
+		) {
+  my $method = $name;
+  Entity $method => sub {
+    shift;
+    $CON->$method(@_);
+  };
+}
+
+foreach my $item (map([$_ => uc($_)]
+		      , qw/path_info
+			   request_uri
+
+			   script_uri
+			   script_url
+			   script_filename
+
+                           SCRIPT_NAME
+			   /)) {
+  my ($method, $env_name) = @$item;
+  Entity $method => sub {
+    my Env $env = $CON->env;
+    $env->{$env_name};
+  };
+}
+
+# &yatt:SCRIPT_NAME(); is original $env->{SCRIPT_NAME}
+# &yatt:script_name(); is $env->{'yatt.script_name'}
+
+Entity script_name => sub {
+  my ($this) = @_;
+  my Env $env = $CON->env;
+  $env->{'yatt.script_name'};
+};
+
 #========================================
 
 YATT::Lite::Breakpoint::break_load_dispatcher();
