@@ -7,6 +7,40 @@ use YATT::Lite::WebMVC0::SiteApp; # To make lint happy, this is required.
 
 package YATT::Lite::WebMVC0::SiteApp;
 
+#
+# For debugging only (at least for now).
+#
+sub _callas_cgi {
+  (my MY $self, my $app, my $fh, my Env $init_env, my ($args, %opts)) = @_;
+
+  my Env $env = $self->psgi_cgi_newenv($init_env, $fh);
+
+  my $res;
+  if ($self->{cf_noheader}) {
+    require Cwd;
+    local $env->{GATEWAY_INTERFACE} = 'CGI/YATT';
+    local $env->{REQUEST_METHOD} //= 'GET';
+    local @{$env}{qw(PATH_TRANSLATED REDIRECT_STATUS)}
+      = (Cwd::abs_path(shift @$args), 200)
+      if @_;
+
+    if (my $err = catch { $res = $app->($env) }) {
+      # XXX: Should I do error specific things?
+      $res = $err;
+    }
+
+  } elsif ($env->{GATEWAY_INTERFACE}) {
+    if (my $err = catch { $res = $app->($env) }) {
+      $res = $err;
+    }
+  } else {
+    $res = $app->($env);
+    print terse_dump($res);
+  }
+
+  print terse_dump($res);
+}
+
 sub _runas_cgi {
   (my MY $self, my $fh, my Env $env, my ($args, %opts)) = @_;
   if (-e ".htdebug_env") {
@@ -134,7 +168,10 @@ sub new_cgi {
   my (@params) = do {
     unless (@_) {
       ()
+    } elsif (@_ == 1 and defined $_[0] and not ref $_[0] and $_[0] =~ /[&;]/) {
+      $_[0]; # Raw query string
     } elsif (@_ > 1 or defined $_[0] and not ref $_[0]) {
+      # k=v list
       $self->parse_params(\@_, {})
     } elsif (not defined $_[0]) {
       ();
@@ -222,6 +259,26 @@ sub show_error {
   print $fh "\n". ($error // "");
 }
 
+sub psgi_cgi_newenv {
+  (my MY $self, my Env $init_env, my ($stdin, $stderr)) = @_;
+  require Plack::Util;
+  require Plack::Request;
+  my Env $env = +{ %$init_env };
+  $env->{'psgi.version'} = [1,1];
+  $env->{'psgi.url_scheme'}
+    = ($init_env->{HTTPS}||'off') =~ /^(?:on|1)$/i ? 'https' : 'http';
+  $env->{'psgi.input'}        = $stdin  || *STDIN;
+  $env->{'psgi.errors'}       = $stderr || *STDERR;
+  $env->{'psgi.multithread'}  = &Plack::Util::FALSE;
+  $env->{'psgi.multiprocess'} = &Plack::Util::FALSE; # XXX:
+  $env->{'psgi.run_once'}     = &Plack::Util::FALSE;
+  $env->{'psgi.streaming'}    = &Plack::Util::FALSE; # XXX: Todo.
+  $env->{'psgi.nonblocking'}  = &Plack::Util::FALSE;
+  # delete $env->{HTTP_CONTENT_TYPE};
+  # delete $env->{HTTP_CONTENT_LENGTH};
+  $env;
+}
+
 &YATT::Lite::Breakpoint::break_load_dispatcher_cgi;
 
 1;
